/**
 * @file 百度文库表格处理js逻辑
 */
var currentProcessPage = 1;
var fontSizeLevels = {};
var styleCache = [];

var Ctor = function () {};
var U = {

    encodeHtml: function (text) {
        return $('<div/>').text(text).html().replace(/\s{2}/g, '&nbsp; ');
    },
    host: 'https://wkrtcs.bdimg.com',

    inherits: function (parent, protoProps, staticProps) {
        var child;

        if (protoProps && protoProps.hasOwnProperty('constructor')) {
            child = protoProps.constructor;
        }
        else {
            child = function () {
                parent.apply(this, arguments);
            };
        }

        $.extend(child, parent);

        Ctor.prototype = parent.prototype;
        child.prototype = new Ctor();

        if (protoProps) {
            $.extend(child.prototype, protoProps);
        }

        if (staticProps) {
            $.extend(child, staticProps);
        }

        child.prototype.constructor = child;

        child._super = parent.prototype;

        return child;
    }
};

var util = {
    limitImg: function (params) {
        var w = params.w;
        var h = params.h;
        var percent = h / w;
        var limit = 3000;

        // 如果未中小流量，则使用原有线上策略 全流量上线
        // if (!smallFlowImg()) {
        //     return {
        //         width: parseInt(params.width, 10),
        //         height: parseInt(percent * params.width, 10)
        //     };
        // }

        // 如果图片较大阈值为1080000，则使用原图一半
        if (h * w > 1080000) {
            w = params.w / 2;
            h = percent * w;
        }

        // 图片最大阈值限制为宽3000 高3000
        if (w > limit) {
            w = limit;
            h = percent * w;
        }

        if (h > limit) {
            h = limit;
            w = h / percent;
        }

        return {
            width: parseInt(w, 10),
            height: parseInt(h, 10)
        };
    }
};

var globalData = {};
globalData.set = function () {
    var args = arguments;
    if (args.length === 1 && typeof args[0] === 'object') {
        $.extend(this, args[0]);
    }
    else if (args.length === 2 && typeof args[0] === 'string') {
        this[args[0]] = args[1];
    }

};

function SectionProcessor() {
}

SectionProcessor.prototype = {
    constructor: SectionProcessor,
    // abstract method, process json to html
    process: function (source) {
        if (!this.checkSource()) {
            return false;
        }

        return source;
    },

    // abstract method, check whether the resource is OK
    checkSource: function () {
        return true;
    },

    // Process Class Attribute of Element
    processClass: function (classArr) {
        if (!$.isArray(classArr)) {
            return arguments.callee.call(this, [classArr]);
        }

        var output = [];

        $(classArr).each(function (index, className) {
            output.push('rtcscls' + currentProcessPage + '_' + className);
        });

        for (var i = 0, l = output.length; i < l; i++) {
            if (fontSizeLevels['.' + output[i]]) {
                output.push('fszl' + fontSizeLevels['.' + output[i]]);
            }

        }

        return output.join(' ');
    }
};

SectionProcessor.setCurrentPage = function (num) {
    currentProcessPage = num;
};

var TableProcessor = U.inherits(SectionProcessor, {

    constructor: function () {
        SectionProcessor.apply(this, arguments);

        this.__attrArr = ['colspan', 'rowspan'];
    },

    process: function (source, cb) {
        if (!this.checkSource(source)) {
            return false;
        }

        var output = '<div class="tableDiv" id="tab' + new Date().getTime() + '"><table';
        for (var attr in source) {
            if (!source.hasOwnProperty(attr)) {
                continue;
            }

            switch (attr) {
                case 'r':
                    output += ' class="' + this.processClass(source.r) + '"';
                    break;
                case 'cellpadding':
                case 'cellspacing':
                    output += ' ' + attr + '=' + source[attr];
                    break;
            }
        }
        output += '>';
        var me = this;

        $(source.c).each(function (index, tr) {
            if (!me.checkSource(tr)) {
                return false;
            }

            output += '<tr ' + (tr.r ? 'class="' + me.processClass(tr.r) + '"' : '') + '>';
            $(tr.c).each(function (index, td) {
                output += '<td';

                for (var attr in td) {
                    if (!td.hasOwnProperty(attr)) {
                        continue;
                    }

                    switch (attr) {
                        case 'r':
                            output += ' class="' + me.processClass(td.r) + '"';
                            break;
                        case 'colspan':
                        case 'rowspan':
                            output += ' ' + attr + '=' + td[attr];
                            break;
                    }
                }

                output += '>'
                    + (td.c ? cb($.isArray(td.c) ? td.c : [td.c]) : '')
                    + '</td>';
            });
            output += '</tr>';
        });

        output += '</table></div>';
        return output;
    },

    checkSource: function (source) {
        return source.c && $.isArray(source.c) && source.c.length;
    }
});

var tableProcessor = new TableProcessor();

var ParagraphProcessor = U.inherits(SectionProcessor, {
    process: function (source, inner) {
        var output = '';
        var me = this;
        var cls = !inner && source.r ? this.processClass(source.r) : false;
        var firstP = true;

        if (!source.c || !source.c.length || (typeof source.c === 'string' && $.trim(source.c) === '')) {
        }
        else {
            output += inner ? '' : '<' + source.t + (cls ? ' class="' + cls + '"' : '') + '>';
            $(source.c).each(function (index, item) {
                switch (item.t) {
                    case 'img':
                        output += imageProcessor.process(item);
                        break;
                        // case 'a':
                        //     output += linkProcessor.process(item);
                        //     break;
                        // case 'br':
                        //     output += breakProcessor.process(item);
                        //     break;
                        // case 'ruby':
                        //     output += rubyProcessor.process(item);
                        //     break;
                        // case 'obj':
                        //     output += objProcessor.process(item);
                        //     break;
                    default:
                        if (firstP) {
                            output += textProcessor.process(item, me, firstP);
                            firstP = false;
                        }
                        else {
                            output += textProcessor.process(item, me);
                        }
                }
            });
            output += inner ? '' : '</' + source.t + '>';
        }

        return output;
    },

    checkSource: function () {
        return true;
    }
});

var paragraphProcessor = new ParagraphProcessor();

var TextProcessor = U.inherits(SectionProcessor, {
    process: function (source, paragraphProcessor, firstP) {

        if (undefined === source.c || typeof (source) === 'string') {
            return source;
        }

        var cls = source.r ? this.processClass(source.r) : false;

        // 屏蔽链接的特殊样式
        if (source.dataType === 'hyperlink') {
            cls += ' hyperlink';
        }

        if (source.dataType === 'BN' && !$.isArray(source.c)) {
            cls += ' BN';
        }

        if ($.isArray(source.c)) {
            return '<' + source.t + (cls ? ' class="' + cls + '"' : '') + '>'
                + paragraphProcessor.process(source, true)
                + '</' + source.t + '>';
        }

        if (firstP) {
            if (!cls) {
                return U.encodeHtml(source.c);
            }

            return '<' + source.t + ' class="' + cls + '">' + U.encodeHtml(source.c) + '</' + source.t + '>';
        }

        if (!cls) {
            return U.encodeHtml(source.c);
        }

        return '<' + source.t + ' class="' + cls + '">' + U.encodeHtml(source.c) + '</' + source.t + '>';

    },

    checkSource: function () {
        return true;
    }
});

var textProcessor = new TextProcessor();

var ImageProcessor = U.inherits(SectionProcessor, {
    process: function (source) {
        if (!source || !source.w || !source.c) {
            return '';
        }

        if (source.dataType === 'vml' || source.dataType === 'emf') {
            return '';
        }

        var width = source.w ? source.w : false;
        var height = source.h ? source.h : false;
        var md5sum = globalData.md5sum;
        var sign = globalData.sign;
        var ipr = encodeURIComponent(JSON.stringify(source));
        var bucketNum = globalData.bucketNum;
        // 业务整体逻辑此处容器尚未插入到页面中
        var containerPadding = 17;
        var limitWidth = $('body').width() - containerPadding * 2;
        var output = '';
        var offlineSrc = '';
        var offlineSrcIOS = source.c.split('/')[source.c.split('/').length - 1];
        var isIOS = /\(i[^;]+;( U;)? CPU.+Mac OS X/.test(navigator.userAgent);

        if (/(.png|.jpeg|.gif|.bmp")$/.test(source.c)) {
            offlineSrc = source.c.replace(new RegExp('/', 'g'), '_');
        }
        else {
            var offlineImg = source.c.split('.')[0];
            offlineSrc = offlineImg.replace(new RegExp('/', 'g'), '_') + '.png';
        }

        if (!source.w) {
            if (globalData.offlinePath) {
                if (isIOS) {
                    output = '<img class="' + source.r + '" src="'
                        + globalData.offlinePath + offlineSrcIOS + '" style="height: 1em;" />';
                }
                else {
                    output = '<img class="' + source.r + '" src="'
                        + globalData.offlinePath + offlineSrc + '" style="height: 1em;" />';
                }
            }
            else {
                output = '<img class="' + source.r + '" src="' + U.host
                    + '/rtcs/image?w=' + width + '&md5sum=' + md5sum + '&sign=' + sign
                    + '&rtcs_flag=1&l=webapp&bucketNum='
                    + bucketNum + '&ipr=' + ipr + '" style="height: 1em;" />';
            }
        }
        else if (source.w > limitWidth) {
            var limitHeight = parseInt(source.h / source.w * limitWidth, 10);
            // 图片优化 - 规则见util.limitImg方法, 目前小流量上线
            width = util.limitImg({
                w: source.w,
                h: source.h,
                width: 300
            }).width;

            if (globalData.offlinePath) {
                if (isIOS) {
                    output = '<p class="' + this.processClass(source.r) + ' bigPictures"><img class="'
                        + source.r + '" src="' + globalData.offlinePath + offlineSrcIOS
                        + '" class="' + this.processClass(source.r) + '" width='
                        + limitWidth + ' height=' + limitHeight + ' style="width:'
                        + limitWidth + 'px;height:auto;" /></p>';
                }
                else {
                    output = '<p class="' + this.processClass(source.r) + ' bigPictures"><img class="'
                        + source.r + '" src="' + globalData.offlinePath + offlineSrc
                        + '" class="' + this.processClass(source.r) + '" width='
                        + limitWidth + ' height=' + limitHeight + ' style="width:'
                        + limitWidth + 'px;height:auto;" /></p>';
                }
            }
            else {
                output = '<p class="' + this.processClass(source.r) + ' bigPictures"><img class="'
                    + source.r + '" src="' + U.host + '/rtcs/image?w='
                    + width + '&md5sum=' + md5sum + '&sign=' + sign
                    + '&rtcs_flag=1&l=webapp&bucketNum=' + bucketNum + '&ipr=' + ipr
                    + '" class="' + this.processClass(source.r) + '" width='
                    + limitWidth + ' height=' + limitHeight + ' style="width:'
                    + limitWidth + 'px;height:auto;" /></p>';
            }
        }
        else {
            var size = '';
            if (width && height) {
                size = 'width="' + width + '" height="' + height + '"';
            }

            if (globalData.offlinePath) {
                if (isIOS) {
                    output = '<img ' + size + ' class="' + this.processClass(source.r) + '" src="'
                        + globalData.offlinePath + offlineSrcIOS + '" />';
                }
                else {
                    output = '<img ' + size + ' class="' + this.processClass(source.r) + '" src="'
                        + globalData.offlinePath + offlineSrc + '" />';
                }
            }
            else {
                output = '<img ' + size + ' class="' + this.processClass(source.r) + '" src="'
                    + U.host + '/rtcs/image?w=' + width + '&md5sum=' + md5sum + '&sign=' + sign
                    + '&rtcs_flag=1&l=webapp&bucketNum=' + bucketNum + '&ipr=' + ipr + '" />';
            }
        }

        return output;
    },
    rotateFunc: function (data) {
        var re = this.preFix() + 'transform: ';
        if (!data.rot && !data.fh && !data.fv) {
            return;
        }

        if (data.rot) {
            re += 'rotate(' + data.rot + 'deg) ';
        }

        if (data.fv && data.fh) {
            re += ' Matrix(-1,0,0,-1,0,0)';
        }
        else if (data.fv) {
            re += ' Matrix(1,0,0,-1,0,0)';
        }
        else if (data.fh) {
            re += ' Matrix(-1,0,0,1,0,0)';
        }

        return re;

    },

    preFix: function () {
        var bro = $.support;
        var ua = navigator.userAgent.toLowerCase();
        if (/firefox/.test(ua)) { // FireFox
            return '-moz-';
        }
        else if (/chrome/.test(ua) || /safari/.test(ua)) { // Chrome or Safari
            return '-webkit-';
        }
        else if (bro.boxModel && bro.leadingWhitespace) { // IE9
            return '-ms-';
        }

        return '';
    },
    checkSource: function () {
        return true;
    }
});

var imageProcessor = new ImageProcessor();

function processEntry(contentArr, opt) {
    var ret = '';

    $.each(contentArr, function (index, contentItem) {
        if (!contentItem.t) {
            contentItem.t = 'p';
        }

        switch (contentItem.t) {
            // paragraph
            case 'p':
            case 'h1':
            case 'h2':
            case 'h3':
            case 'h4':
            case 'h5':
            case 'h6':
            case 'h7':
            case 'h8':
            case 'h9':
            case 'h10':
                ret += processParagraph(contentItem);
                break;
                // table
            case 'table':
                ret += processTable(contentItem);
                break;
                // list
                // case 'ol':
                // case 'ul':
                //     ret += processList(contentItem, contentItem.t);
                //     break;
        }
    });

    return ret;
}

// Process paragraph
function processParagraph(p) {
    return paragraphProcessor.process(p);
}

// Process List
// function processList(list, type) {
//     return listProcessor.process(list, type);
// }

// Process Table
function processTable(trs) {
    return tableProcessor.process(trs, processEntry);
}

// Process Common Styles, these styles are defined in style object
function processStyle(styleObj, divClass) {
    var cssText = '';
    var className;
    for (className in styleObj) {
        if (!styleObj.hasOwnProperty(className)) {
            continue;
        }

        var cssStr = filterCss(styleObj[className], '.' + getClassPrefix() + className);
        cssText += '.' + divClass + ' .' + getClassPrefix() + className + '{' + cssStr + '}';
    }

    // 背景颜色@yangguang23
    if (globalData.backgroundColor) {
        cssText += 'body {background-color: ' + globalData.backgroundColor + '}';
    }

    var $style = $('<style>' + cssText + '</style>');

    styleCache.push($style);
    $style.appendTo($('body').find('.style-head'));
}

// Process Defautl Style, these styles are defined in baseStyle object, and refer to Common Style.
function processDefaultStyle(rawStyleObj, styleObj, divClass) {
    var cssText = '';
    var type;

    for (type in styleObj) {
        if (!styleObj.hasOwnProperty(type)) {
            continue;
        }

        $(styleObj[type]).each(function (index, item) {
            var cssStr = filterCss(rawStyleObj[item], type);
            cssText += '.' + divClass + ' ' + type + '{' + cssStr + '}';
        });
    }

    var $style = $('<style>' + cssText + '</style>');
    styleCache.push($style);
    $style.appendTo($('body').find('.style-head'));
}

// filter css match wap sets
function filterCss(t, child) {
    if (!t || typeof t !== 'object') {
        return;
    }

    var csss = t;
    var cssStr = '';
    // 是否为上下标样式
    var isSupscript = t && t['vertical-align'] && (t['vertical-align'] === 'super' || t['vertical-align'] === 'sub');
    for (var i in csss) {
        if (!csss.hasOwnProperty(i)) {
            continue;
        }

        var styleName = i;
        var styleValue = csss[i];
        if (/text-indent/.test(styleName)
            || /font-size/.test(styleName)) {
        // 去除首行缩进
        // 去掉字号大小，统一18px
        }
        // 上下标 设置样式 及 字体
        else if (/vertical-align/.test(styleName)) {
            cssStr += 'vertical-align:' + styleValue + '!important;';
            // 上下标 处理字体大小为正常字体大小的一半
            if (isSupscript) {
                fontSizeLevels[child] = '_half';
            }
        }
        else if (/line-height/.test(styleName)) {
            if (/px/.test(styleValue)) {
                if (parseInt(styleValue, 10) > 60) {
                    cssStr += 'line-height: 60px';
                }
                else {
                    cssStr += (styleName + ':' + styleValue + ';');
                }
            }
            else if (/\%/.test(styleValue)) {
                if (parseInt(styleValue, 10) > 200) {
                    cssStr += 'line-height: 200%';
                }
                else {
                    cssStr += (styleName + ':' + styleValue + ';');
                }
            }
        }
        else {
            cssStr += (styleName + ':' + styleValue + ';');
        }
    }
    return cssStr;
}

// Get CSS Class prefix
function getClassPrefix() {
    return 'rtcscls' + currentProcessPage + '_';
}

function rePosTablesNew() {

    var tables = $('body').find('.rtcscontent > .tableDiv');
    var rtcscontent = $('body').find('.rtcscontent');
    if (!rtcscontent.length) {
        return;
    }

    // 表格可视区域最大宽度
    var padding = parseFloat(rtcscontent.css('padding-left')) * 2;
    // var maxWidth = $('body').width() - padding - 12;
    var maxWidth = $('body').width() - padding;

    for (var i = 0, l = tables.length; i < l; i++) {
        var tableWrap = $(tables[i]);
        var table = $(tableWrap.find('table')[0]);
        tableWrap.addClass('tablenew').css('width', maxWidth + 'px');
        tablePreScale(tableWrap, table, maxWidth);
    }
}

function tablePreScale(tableWrap, table, maxWidth) {
    var me = this;
    var imgs = table.find('img');
    var needNum = 0;
    // 表格里如果有图片，需要等图片加载完后 进行缩放
    if (imgs.length) {
        var handle = function () {
            if (!--needNum) {
                me.tableScale(tableWrap, table, maxWidth);
                $(this).off('load error', handle);
            }

        };
        for (var i = 0, l = imgs.length; i < l; i++) {
            if (!imgs[i].complete) {
                needNum++;
                $(imgs[i]).on('load error', handle);
            }

        }
    }

    tableScale(tableWrap, table, maxWidth);
}

function tableScale(tableWrap, table, maxWidth) {
    var isIOS = /\(i[^;]+;( U;)? CPU.+Mac OS X/.test(navigator.userAgent);
    table.css({
        '-webkit-transform': 'none',
        'transform': 'none'
    });
    // 若table为隐藏元素，则获取的宽高为0，此处进行处理，获取表格宽高
    // var isHide = table.is(':hidden');
    var isHide = false;
    var hideParent = $('body');
    var hideCss = {
        position: hideParent.css('position'),
        display: hideParent.css('display'),
        visibility: hideParent.css('visibility')
    };
    if (isHide) {
        hideParent.css({
            position: 'absolute',
            display: '',
            visibility: 'hidden'
        });
    }

    var width = table.width();
    var height = table.height();
    if (isHide) {
        hideParent.css(hideCss);
    }

    var scale;
    var wrapWidth;

    if (width > maxWidth) {
        scale = maxWidth / width;
        wrapWidth = maxWidth;
    }
    else {
        scale = 1;
        wrapWidth = width;
    }

    tableWrap.css({
        width: wrapWidth + 'px',
        height: height * scale + 'px'
    });

    // table 高度小于页面高度时垂直居中@yangguang23
    if (height * scale < window.screen.height && globalData.isFullscreen) {
        tableWrap.css({
            'margin-top': Math.ceil((window.innerHeight - height * scale) / 2) + 'px'
        });
    }

    // 传递表格高度@yangguang23
    if (isIOS) {
        var iframe = document.createElement('iframe');
        iframe.src = 'wenku://table?height=' + height * scale;
        iframe.style.display = 'none';
        document.documentElement.appendChild(iframe);
    }
    else {
        prompt('wenku://table?height=' + height * scale);
    }

    if (height * scale < 60) {
        this.tableHeightLess60++;
    }

    table.css({
        '-webkit-transform': 'scale(' + scale + ')',
        'transform': 'scale(' + scale + ')'
    });
}

function buildDiv(json, style, baseStyle, md5sum, sign, bucketNum, backgroundColor, offlinePath, isFullscreen) {
    globalData.set({
        md5sum: md5sum,
        sign: sign,
        bucketNum: bucketNum,
        backgroundColor: backgroundColor,
        offlinePath: offlinePath,
        isFullscreen: isFullscreen
    });

    var contentArr = [];
    contentArr.push(json);

    var table = processEntry(contentArr);

    $('.table').html(table);

    if ($.type(style) === 'object') {
        processStyle(style, 'content');
        processDefaultStyle(style, baseStyle, 'content');
    }

    rePosTablesNew();
}

window.buildDiv = buildDiv;